@@ -10,6 +10,7 @@ from collections.abc import Iterable
 import glob
 import datetime
 import shutil
+from .dictionary_backends import TinyDictionary, MongoDictionary
 
 
 try:
@@ -77,7 +78,7 @@ def __where_models(language, safe=False, return_latest=True):
 				t = _file_modification_time(files[0])
 				if latest_model[0] is None or t > latest_model[1]:
 					latest_model = [path, t]
-	if return_latest:
+	if return_latest and latest_model[0] is not None:
 		return latest_model[0]
 	if safe:
 		return None
@@ -96,7 +97,7 @@ def uninstall(language):
 		path = __where_models(language, safe=True)
 
 def download(language, show_progress=True):
-	model_types = {"analyser":"analyser-gt-desc.hfstol", "analyser-norm":"analyser-gt-norm.hfstol", "generator-desc":"generator-gt-desc.hfstol", "generator-norm":"generator-gt-norm.hfstol", "generator":"generator-dict-gt-norm.hfstol", "cg":"disambiguator.bin", "metadata.json":"metadata.json"}
+	model_types = {"analyser":"analyser-gt-desc.hfstol", "analyser-norm":"analyser-gt-norm.hfstol", "generator-desc":"generator-gt-desc.hfstol", "generator-norm":"generator-gt-norm.hfstol", "generator":"generator-dict-gt-norm.hfstol", "cg":"disambiguator.bin", "metadata.json":"metadata.json", "dictionary.json":"dictionary.json"}
 	download_to = os.path.join(__find_writable_folder(__model_base_folders()), language)
 	ssl._create_default_https_context = ssl._create_unverified_context
 	if not os.path.exists(download_to):
@@ -115,6 +116,7 @@ def download(language, show_progress=True):
 
 generator_cache = {}
 analyzer_cache = {}
+dictionary_cache = {}
 
 def __generator_model_name(descrpitive, dictionary_forms):
 	if not descrpitive and dictionary_forms:
@@ -290,7 +292,8 @@ def lemmatize(word, language, force_local=True, descrpitive=True, word_boundarie
     return lemmas
 
 def supported_languages():
-	return __send_request("listLanguages/", {"user": "uralicApi"})
+	d = requests.get("https://uralic.mikakalevi.com/nightly/supported_languages.json")
+	return d.json()
 
 def __api_analyze(word, language,descrpitive=True):
 	if not descrpitive:
@@ -302,9 +305,35 @@ def __api_generate(query, language, descrpitive=False, dictionary_forms=True):
 		raise UnsupportedModel("Server only supports normative dictionary forms. Please download the models and generate locally")
 	return __send_request("generate/", {"query": query, "language": language})["analysis"]
 
-def dictionary_search(word, language):
+def dictionary_search(word, language, force_local=True, backend=TinyDictionary):
+	if force_local:
+		d = _get_dictionary(language,backend=backend)
+		lemmas = list(set(lemmatize(word, language)) - set([word]))
+		return d.find(word, lemmas)
+	else:
+		return _api_dictionary_search(word, language)
+
+def dictionary_lemmas(language, backend=TinyDictionary):
+	d = _get_dictionary(language,backend=backend)
+	return d.lemmas()
+
+def import_dictionary_to_db(language, backend=MongoDictionary):
+	d = _get_dictionary(language, backend=backend)
+	d.import_data()
+
+def _api_dictionary_search(word, language):
 	return __send_request("search/", {"word": word, "language": language})
 
 def __send_request(url, data):
 	r = requests.get(api_url + url, params=data)
 	return r.json()
+
+def _get_dictionary(language, backend=TinyDictionary):
+	cache_key = backend.__name__ + "_" + language
+	if cache_key in dictionary_cache:
+		return dictionary_cache[cache_key]
+	path = os.path.join(__where_models(language), "dictionary.json")
+	dictionary = backend(path, language)
+	dictionary_cache[cache_key] = dictionary
+	return dictionary
+
